#
# Copyright (c) 2003 Art Haas
#
# This file is part of PythonCAD.
#
# PythonCAD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PythonCAD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PythonCAD; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# quadtree class definition
#

class Quadtree(object):
    def __init__(self):
        self.__parent = None
        self.__swchild = None
        self.__sechild = None
        self.__nwchild = None
        self.__nechild = None
        self.__xmin = None
        self.__ymin = None
        self.__xmax = None
        self.__ymax = None
        self.__threshold = 5
        self.__objects = []

    def setBoundary(self, xmin, ymin, xmax, ymax):
        _xmin = xmin
        if not isinstance(_xmin, float):
            _xmin = float(xmin)
        _ymin = ymin
        if not isinstance(_ymin, float):
            _ymin = float(ymin)
        _xmax = xmax
        if not isinstance(_xmax, float):
            _xmax = float(xmax)
        if _xmax < _xmin:
            raise ValueError, "Illegal values: xmax < xmin"
        _ymax = ymax
        if not isinstance(_ymax, float):
            _ymax = float(ymax)
        if _ymax < _ymin:
            raise ValueError, "Illegal values: ymax < ymin"
        self.__xmin = _xmin
        self.__ymin = _ymin
        self.__xmax = _xmax
        self.__ymax = _ymax

    def getBoundary(self):
        return self.__xmin, self.__ymin, self.__xmax, self.__ymax

    def setThreshold(self, count):
        _count = count
        if not isinstance(_count, int):
            _count = int(_count)
        if _count < 1:
            raise ValueError, "Invalid threshold: %d" % _count
        self.__threshold = _count

    def getThreshold(self):
        return self.__threshold

    def setParent(self, parent):
        if not isinstance(parent, Quadtree):
            raise TypeError, "Invalid Quadtree parent: " + `parent`
        self.__parent = parent

    def getParent(self):
        return self.__parent

    def addObject(self, obj):
        _seen = False
        for _obj in self.__objects:
            if obj is _obj:
                _seen = True
                break
        if not _seen:
            self.__objects.append(obj)
        return self.__threshold > len(self.__objects)

    def getObjects(self):
        return self.__objects[:]

    def delObjects(self):
        del self.__objects[:]

    def getChildren(self):
        return self.__swchild, self.__sechild, self.__nwchild, self.__nechild
    
    def split(self):
        _xmin = self.__xmin
        _ymin = self.__ymin
        _xmax = self.__xmax
        _ymax = self.__ymax
        _xmid = (_xmin + _xmax)/2.0
        _ymid = (_ymin + _ymax)/2.0
        _count = self.__threshold
        #
        # SW Child
        #
        _child = Quadtree()
        _child.setParent(self)
        _child.setThreshold(_count)
        _child.setBoundary(_xmin, _ymin, _xmid, _ymid)
        self.__swchild = _child
        #
        # SE Child
        #
        _child = Quadtree()
        _child.setParent(self)
        _child.setThreshold(_count)
        _child.setBoundary(_xmid, _ymin, _xmax, _ymid)
        self.__sechild = _child
        #
        # NW Child
        #
        _child = Quadtree()
        _child.setParent(self)
        _child.setThreshold(_count)
        _child.setBoundary(_xmin, _ymid, _xmax, _ymax)
        self.__nwchild = _child
        #
        # NE Child
        #
        _child = Quadtree()
        _child.setParent(self)
        _child.setThreshold(_count)
        _child.setBoundary(_xmid, _ymid, _xmax, _ymax)
        self.__nwchild = _child
